@@ -3,7 +3,6 @@ use crate::ast::{
 };
 use syn::parse::{Parse, ParseStream, Result};
 use syn::punctuated::Punctuated;
-use syn::Ident;
 
 impl Parse for Struct {
     fn parse(input: ParseStream) -> Result<Self> {
@@ -12,18 +11,12 @@ impl Parse for Struct {
         let name = input.parse()?;
         let generics = input.parse()?;
         let fields = {
-            let lookahead = input.lookahead1();
-            if lookahead.peek(syn::token::Paren) {
+            if input.peek(syn::token::Paren) {
                 let fields;
                 syn::parenthesized!(fields in input);
-                Punctuated::<(syn::Ident, syn::Type), syn::token::Comma>::parse_terminated_with(
+                Punctuated::<syn::Field, syn::token::Comma>::parse_terminated_with(
                     &fields,
-                    |inner| {
-                        let name = inner.parse()?;
-                        let _: syn::Token![:] = inner.parse()?;
-                        let ty = inner.parse()?;
-                        Ok((name, ty))
-                    },
+                    syn::Field::parse_named,
                 )?
                 .into_pairs()
                 .map(|pair| pair.into_value())
@@ -73,6 +66,7 @@ impl Parse for Node {
         if lookahead.peek(syn::Ident)
             || lookahead.peek(syn::Token![#])
             || lookahead.peek(syn::Token![.])
+            || lookahead.peek(syn::Token![$])
         {
             Ok(Node::Element(input.parse()?))
         } else if lookahead.peek(syn::Token![@]) {
@@ -87,14 +81,9 @@ impl Parse for Node {
             } else if lookahead.peek(syn::token::Match) {
                 let _: syn::token::Match = input.parse()?;
                 Ok(Node::Match(input.parse()?))
-            } else if lookahead.peek(syn::Lit) {
-                Ok(Node::Expr(input.parse()?))
-            } else if let Ok(syn::Expr::Call(_))
-            | Ok(syn::Expr::Field(_))
-            | Ok(syn::Expr::Macro(_))
-            | Ok(syn::Expr::MethodCall(_))
-            | Ok(syn::Expr::Path(_))
-            | Ok(syn::Expr::Struct(_)) = input.fork().parse::<syn::Expr>()
+            } else if lookahead.peek(syn::Lit)
+                || lookahead.peek(syn::Ident)
+                || lookahead.peek(syn::token::Brace)
             {
                 Ok(Node::Expr(input.parse()?))
             } else if input.fork().parse::<syn::Stmt>().is_ok() {
@@ -104,11 +93,9 @@ impl Parse for Node {
             }
         } else if lookahead.peek(syn::Lit) {
             let lit: syn::Lit = input.parse()?;
-            Ok(Node::Expr(syn::parse2(quote::quote!(#lit))?))
+            Ok(Node::Expr(syn::parse_quote!(#lit)))
         } else if lookahead.peek(syn::token::Brace) {
-            let inner;
-            syn::braced!(inner in input);
-            Ok(Node::Expr(inner.parse()?))
+            Ok(Node::Expr(input.parse()?))
         } else {
             Err(lookahead.error())
         }
@@ -119,20 +106,24 @@ impl Parse for Element {
     fn parse(input: ParseStream) -> Result<Self> {
         let (name, mut id, mut classes) = {
             let lookahead = input.lookahead1();
-            if lookahead.peek(syn::Ident) {
-                let name: Ident = input.parse()?;
-                (name.to_string(), None, Vec::new())
+            if lookahead.peek(syn::Token![$]) {
+                let _: syn::Token![$] = input.parse()?;
+                let name = identifier_or_string_literal_or_expression(input)?;
+                (name, None, Vec::new())
+            } else if lookahead.peek(syn::Ident) {
+                let name = identifier_or_string_literal_or_expression(input)?;
+                (name, None, Vec::new())
             } else if lookahead.peek(syn::Token![#]) {
                 let _: syn::Token![#] = input.parse()?;
                 (
-                    "div".into(),
+                    syn::parse_quote!("div"),
                     Some(identifier_or_string_literal_or_expression(input)?),
                     Vec::new(),
                 )
             } else if lookahead.peek(syn::Token![.]) {
                 let _: syn::Token![.] = input.parse()?;
                 (
-                    "div".into(),
+                    syn::parse_quote!("div"),
                     None,
                     vec![identifier_or_string_literal_or_expression(input)?],
                 )
@@ -142,11 +133,10 @@ impl Parse for Element {
         };
 
         loop {
-            let lookahead = input.lookahead1();
-            if lookahead.peek(syn::Token![#]) {
+            if input.peek(syn::Token![#]) {
                 let _: syn::Token![#] = input.parse()?;
                 id = Some(identifier_or_string_literal_or_expression(input)?);
-            } else if lookahead.peek(syn::Token![.]) {
+            } else if input.peek(syn::Token![.]) {
                 let _: syn::Token![.] = input.parse()?;
                 classes.push(identifier_or_string_literal_or_expression(input)?);
             } else {
@@ -155,8 +145,7 @@ impl Parse for Element {
         }
 
         let attributes = {
-            let lookahead = input.lookahead1();
-            if lookahead.peek(syn::token::Bracket) {
+            if input.peek(syn::token::Bracket) {
                 let attributes;
                 syn::bracketed!(attributes in input);
                 Punctuated::<Attribute, syn::Token![,]>::parse_terminated(&attributes)?
@@ -201,11 +190,9 @@ impl Parse for If {
         let mut clauses = vec![input.parse()?];
         let mut default = None;
         loop {
-            let lookahead = input.lookahead1();
-            if lookahead.peek(syn::token::Else) {
+            if input.peek(syn::token::Else) {
                 let _: syn::token::Else = input.parse()?;
-                let lookahead = input.lookahead1();
-                if lookahead.peek(syn::token::If) {
+                if input.peek(syn::token::If) {
                     let _: syn::token::If = input.parse()?;
                     clauses.push(input.parse()?);
                 } else {
@@ -238,8 +225,7 @@ impl Parse for IfClause {
 
 impl Parse for IfClauseTest {
     fn parse(input: ParseStream) -> Result<Self> {
-        let lookahead = input.lookahead1();
-        if lookahead.peek(syn::token::Let) {
+        if input.peek(syn::token::Let) {
             let _: syn::token::Let = input.parse()?;
             let pattern = input.parse()?;
             let _: syn::Token![=] = input.parse()?;
@@ -341,16 +327,11 @@ impl<P: Parse> Parse for Many<P> {
 }
 
 fn identifier_or_string_literal_or_expression(input: ParseStream) -> Result<syn::Expr> {
+    use syn::ext::IdentExt;
     let lookahead = input.lookahead1();
-    let ident = input.step(|cursor| {
-        if let Some((ident, rest)) = cursor.ident() {
-            Ok((ident, rest))
-        } else {
-            Err(cursor.error("expected identifier".to_string()))
-        }
-    });
-    if let Ok(ident) = ident {
-        let string = ident.to_string();
+    if lookahead.peek(syn::Ident::peek_any) {
+        let ident = syn::Ident::parse_any(input)?;
+        let string = ident.unraw().to_string();
         Ok(syn::parse_quote!(#string))
     } else if lookahead.peek(syn::LitStr) {
         let string = input.parse::<syn::LitStr>()?.value();
